#ifndef DISCRETEDATASET_H
#define DISCRETEDATASET_H

# include <QVector>
# include <QString>
# include <QFile>
# include <QTextStream>

typedef QStringList Pattern;

class DiscreteDataset
{
private:
    /**
     * @brief header, ta onomata ton katigorion. Stin teleytaia thesi einai i exodos
     */
    Pattern          header;//ta onomata ton metabliton
    /**
     * @brief xpoint, einai pinakas me ta protypa
     */
    QVector<Pattern> xpoint;//ta x dedomena (protypa)
    /**
     * @brief ypoint, pinakas me tis epithymities klaseis ana protypo
     */
    Pattern          ypoint;//ta y dedomena (katigories)
    /**
     * @brief classVector, o pinakas me tis monadikes katigories
     */
    Pattern          classVector;//oi monadikes katigories

    /**
     * @brief makeClass, dimioyrgei ton pinaka me tis monadikes katigories
     */
    void    makeClass();
public:
    /**
     * @brief DiscreteDataset, fortonei ena dataset apo arxeio. To
     * arxeio einai se morfi csv.
     * @param filename
     */
    DiscreteDataset(QString filename);
    /**
     * @brief DiscreteDataset, dimioyrgei ena dataset me orismata
     * @param h
     * @param xdata
     * @param ydata
     */
    DiscreteDataset(Pattern h,QVector<Pattern>xdata,Pattern ydata);
    /**
     * @brief getheader, epistrefei tin epikefalida
     * @return
     */
    Pattern getheader();
    /**
     * @brief features, epistrefei to plithos ton xaraktiristikon
     * @return
     */
    int         features();
    /**
     * @brief getfeature, epistrefei tin thesi enos xaraktiristikou
     * @param value
     * @return
     */
    int     getfeature(QString value);
    /**
     * @brief enumerateValues, epistrefei oles tis diathesimes times
     * enos xaraktiristikou.
     * @param feature
     * @return
     */
    Pattern enumerateValues(int feature);
    /**
     * @brief getx, epistrefei to protypo stin thesi pos
     * @param pos
     * @return
     */
    Pattern getx(int pos);
    /**
     * @brief gety, epistrefei tin epithymiti katigoria stin thesi pos
     * @param pos
     * @return
     */
    QString     gety(int pos);
    /**
     * @brief getClass, epistrefei tis monadikes katigories
     * @return
     */
    Pattern getClass();
    /**
     * @brief getpoints, epistrefei to plithos ton simeion
     * @return
     */
    int         getpoints();

    /**
     * @brief subDataset, epistrefei to dataset kratontas monon tis times me feature=value
     * @param feature
     * @param value
     * @return
     */
    DiscreteDataset *subDataset(int feature,QString value);
    /**
     * @brief entropy, epistrefei tin entropia tou dataset
     * @return
     */
    double entropy();
    /**
     * @brief gain, epistrefei to information gain enos xaraktiristikou
     * @param feature
     * @return
     */
    double gain(int feature);
    /**
     * @brief featureWithHighestGain, epistrefei to xaraktiristiko me
     * to megalytero information gain
     * @return
     */
    int featureWithHighestGain();
    /**
     * @brief featureWithHighestGain, epistrefei to xaraktiristiko me
     * to megalytero information gain poy den einai mesa sto featureList
     * @param featureList
     * @return
     */
    int featureWithHighestGain(QVector<int> &featureList);
};

#endif // DISCRETEDATASET_H
